<?php

namespace App\Http\Controllers;

use App\Interfaces\IBranch;
use App\Interfaces\ICompany;
use App\Interfaces\IUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;

class BranchController extends Controller
{

    private $branch;

    private $user;

    private $client;
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(IBranch $branch, IUser $user, ICompany $company)
    {
        $this->branch = $branch;
        $this->user = $user;
        $this->client = $company;
        $this->currentTime = Carbon::now();
    }

    public function all()
    {
        $branches = $this->branch->getItem(['status'=> 1], ['id','branch_name', 'slug']);

        return $this->sendResponse($branches, 'success');
    }

    public function activeBranch($client_slug)
    {
        $response = $this->branch->active_branch($client_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);

    }

    public function findBranch($branch_slug)
    {
        $response = $this->branch->get_branch($branch_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }


    public function createBranch(Request $request)
    {

        $validate = Validator::make($request->all(), [
            'branch_name' => 'required|max:191',
            'client_slug'=> 'required',
            'branch_code' => 'required',
            'address' => 'required',
            'contact' => 'required|numeric',
            'alt_contact' => 'nullable|numeric',
            'email' => 'required|email',
            'alt_email' => 'nullable|email',
            'country_id' => 'required|integer',
            'state_id' => 'required|integer',
            'username' => 'required',
            'password' => 'required|confirmed|min:8'
        ]);

        if ($validate->fails()) {
            return $this->sendError("Validation Error - {$validate->errors()->first()}",  $validate->errors());
        }

        $response = $this->branch->addbranch($request->all());



        if ($response->status_code == 200) {
            $client = $this->client->findItem(['client_slug'=> $request->get('client_slug')]);
            $store = $this->branch->create([
                'branch_name' => $response->body->data->branch_name,
                'branch_slug' => $response->body->data->branch_slug,
                'branch_code'=> $response->body->data->branch_code,
                'client_id' => $client->id,
            ]);

            $getsuperusers = $this->user->selectItems(['super_user'=> 1], ['id']);

            $this->user->AddbranchtoSuper($store, $getsuperusers);
        }

        return $this->sendHttpStatusResponse($response->body, $response->status_code);

    }

    public function updateBranch(Request $request, $branch_slug)
    {
        $validate = Validator::make($request->all(), [
            'branch_name' => 'required|max:191',
            'branch_code' => 'required',
            'address' => 'required',
            'contact' => 'required|numeric',
            'alt_contact' => 'nullable|numeric',
            'country_id' => 'required|integer',
            'state_id' => 'required|integer',

        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

        $response = $this->branch->update_branch($request->all(), $branch_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function disableBranch($branch_slug)
    {

        $response =  $this->branch->disable_branch($branch_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function enableBranch($branch_slug)
    {
        $response =  $this->branch->enable_branch($branch_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function branch_token($branch_slug)
    {
        $response = $this->branch->get_branch_token($branch_slug);

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function changePassword(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'new_password'=> 'required',
            'branch_slug'=> 'required'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

        $response = $this->branch->passwordChange($request->all());

        return $this->sendHttpStatusResponse($response->body, $response->status_code);

    }


    public function BranchSurveyReport(Request $request, $client_slug, $survey_id, $branch_slug)
    {
       
        $validate = Validator::make($request->all(), [
            'start_date'=> 'nullable|required',
            'end_date' => 'nullable|required'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

        $response = $this->branch->branchReport($client_slug, $branch_slug, $survey_id,$request->all());

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function RepondentReports(Request $request, $client_slug, $survey_id, $branch_slug)
    {
        $validate = Validator::make($request->all(), [
            'start_date' => 'nullable',
            'end_date' => 'nullable'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

       
        $response = $this->branch->repondentReport($client_slug, $survey_id, $branch_slug, $request->all());

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function adminSummary(Request $request, $client_slug, $survey_id)
    {
        $validate = Validator::make($request->all(), [
            'start_date' => 'nullable',
            'end_date' => 'nullable'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

        $response = $this->branch->adminSummary($client_slug, $survey_id, $request->all());

        return $this->sendHttpStatusResponse($response->body, $response->status_code);

    }

    public function adminReport(Request $request, $client_slug, $survey_id)
    {
        $validate = Validator::make($request->all(), [
            'start_date' => 'nullable',
            'end_date' => 'nullable'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }

        $response = $this->branch->adminReport($client_slug, $survey_id, $request->all());

        return $this->sendHttpStatusResponse($response->body, $response->status_code);
    }

    public function userBranches(Request $request)
    {
        $user = $request->user();

       
        $branches = $this->user->userbranches($user->id);

        return $this->sendSuccessResponse("Success", $branches);
        
    }


}
