<?php

namespace App\Http\Controllers;

use App\Interfaces\IBranch;
use App\Interfaces\IRole;
use App\Interfaces\IUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;

class UserController extends Controller
{

    private $user, $role, $branch;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct(IUser $user, IRole $role, IBranch $branch)
    {
        $this->user = $user;
        $this->role = $role;
        $this->branch = $branch;
    }

    public function index()
    {

        $users = $this->user->listUsers(request()->company->id);
        return $this->sendResponse($users,'');
    }

    public function getUser($id)
    {
        $user = $this->user->getuser($id);

        return $this->sendResponse($user, '');
       
    }

    public function createUser(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "first_name" => "required",
            "last_name" => "required",
            "phone_number" => "required",
            "email" => "required|unique:users,email",
            "password" => "required|min:8",
            "role" => "required|string",
            "branches" => "nullable|array",
            "branches.*" => "nullable|string"
        ]);

        if($validator->fails()){
            return $this->sendError('Validation Error - ' . $validator->errors()->first(), $validator->errors());
        }

        $role = $this->role->findItem(['role_slug'=> $request->get('role')]);

        if(is_null($role)){
            return $this->sendError('Invaild role slug');
        }

        
        
        $user = $this->user->createUser($request->all(), $request->get('password'), $role->id, $request->company->id);

        
        if(!$user){
            return $this->sendError('Error creating user');
        }

        return $this->sendResponse($user, 'user created successfully');

    }


    public function enableUser($id)
    {
        $user = $this->user->findItem(['id'=> $id]);
        if(is_null($user)){
            return $this->sendError('Invalid user id');
        }

        if($user->active_flag == 0){
            $this->user->updateItem(['id'=> $id], ['active_flag'=> 1]);
        }

        $user->active_flag == 1;
        return $this->sendResponse('', 'User enabled successfully');
    }



    public function disableUser($id)
    {
        $user = $this->user->findItem(['id' => $id]);
        if (is_null($user)) {
            return $this->sendError('Invalid user id');
        }

        if ($user->active_flag == 1) {
            $this->user->updateItem(['id' => $id], ['active_flag' => 0]);
        }

        $user->active_flag == 1;
        return $this->sendResponse('', 'User disabled successfully');
    }


    public function forgotPassword(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'email' => 'required|email'
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }
        $user = $this->user->findItem(['email' => $request->get('email')]);

        $date = Carbon::now();
        $expire_time = $date->addMinutes(15)->toDateTimeString();
        // $token = Str::random(128);
        $token = Str::random(64);
        // $token = rand(100001, 999999);

        if (is_null($user)) {
            return $this->sendError('Email not found');
        }

        DB::table('password_resets')->updateOrInsert(
            ['email' => $request->get('email')],
            ['token' => $token, 'expire_time' => $expire_time]
        );

        //Send to Email API - Forgot Password
        try{
            $this->user->sendEmail($request->get('email'), 1, ['token' => env('ADMIN_PASSWORD_RESET') . $token, 'expire_time' => $expire_time]);

        }
        catch(\Exception $ex){
            // Log error
            // return $ex->getMessage();
        }

        return $this->sendResponse('','Email sent');
    }

    public function passwordReset(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'token' => 'required',
            'password' => 'required|min:8|max:24',
        ]);

        if ($validate->fails()) {
            return $this->sendError($validate->errors());
        }
        $token_details = DB::table('password_resets')->where('token', $request['token'])->first();

        $current_date = Carbon::now();

        if(is_null($token_details)){
            return $this->sendError('Token does not exist');
        }
        if($current_date > $token_details->expire_time){
            return $this->sendError('Token has expired');
        }

        $user = $this->user->updateItem(['email'=> $token_details->email], [
            'password'=>  Hash::make($request->get('password'))
        ]);

        if(!$user){
            return $this->sendError('Invalid request');
        }

        return $this->sendResponse($user, 'Password reset done successfully');
    }


    public function changePassword(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'user_id'=> 'required|integer',
            'old_password'=> 'required',
            'password' => 'required|min:8|max:24',
        ]);

        if($validate->fails()){
            return $this->sendError("Validation Error - {$validate->errors()->first()}", $validate->errors());
        }

        $user = $this->user->findItem(['id'=> $request->get('user_id')]);

        if(is_null($user)){
            return $this->sendError('User not found');
        }

        if(!Hash::check($request->get('old_password'), $user->password)){
            return $this->sendError('Invalid old password');
        }

        $user = $this->user->updateItem(['id' => $user->id], [
            'password' =>  Hash::make($request->get('password'))
        ]);

        if (!$user) {
            return $this->sendError('Invalid request');
        }

        return $this->sendResponse($user, 'Password changed successfully');

    }

    public function ToggleBranch($user_id, $branch_id)
    {
        $branch = $this->branch->findItem(['branch_slug'=> $branch_id]);
      
        $response = $this->user->toggleSwitcher($user_id, $branch->id);


        if($response->status != 1){
            return $this->sendError($response->message);
        }

        $user = $this->user->findItem(['id' => $user_id]);
         $data['branches'] = $this->branch->userBranch($user->id);
        return $this->sendResponse($data, 'success');
    }

    public function updateUser(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
           
            'first_name' => 'required|string',
            'last_name' => 'required|string',
            'role' => 'required',
            // 'email' => 'required'
        ]);

        if ($validator->fails()) {
            return $this->sendError("Validation Error - {$validator->errors()->first()}", $validator->errors());
        }

        $user = $this->user->findItem(['id'=> $id]);

        $role = $this->role->findItem(['role_slug' => $request->get('role')]);

        if (is_null($role)) {
            return $this->sendError('Invaild role slug');
        }

        if($request->get('role') == "super-admin"){
            $super_admin = 1;
        }else{
            $super_admin = 0;
        }
        

        $user = $this->user->updateItem(['id'=> $id], [
            'first_name'=> $request->get('first_name'),
            'last_name'=> $request->get('last_name'),
            'role_id'=> $role->id,
            'phone_number'=> $request->get('phone_number'),
            'super_user'=> $super_admin
           
        ]);

        if (!$user) {
            return $this->sendError('Error creating user');
        }

        return $this->sendResponse($user, 'user updated successfully');
    }


    public function userBranches($id)
    {
        $response = $this->user->userbranches($id);

        return $this->sendResponse($response,'');
    }

    public function removeBranches(Request $request, $id)
    {
        $validator = Validator::make($request->all(), [
            "branches" => "required|array",
            "branches.*" => "required|array"

        ]);


        if ($validator->fails()) {
            return $this->sendBadRequestResponse($validator->errors());
        }

       

        $user = $this->user->findItem(['id'=> $id]);

        if(is_null($user)){
            return $this->sendError('User not found');
        }

       
        foreach($request->get('branches') as $key => $value)
        {
            //return $this->sendResponse($key, 'Branches removed successfully');
           $branch =  DB::table('branches')->where('branch_slug', $key)->first();

          $delete =  DB::table('user_branches')->where('user_id', $user->id)
                                            ->where('branch_id', $branch->id)
                                            ->delete();

        }

        return $this->sendResponse($delete, 'Branches removed successfully');
        
    }

}
